#include <Adafruit_SSD1306.h>

#define OLED_I2C_ADDRESS 0x3C
#define OLED_WIDTH 128
#define OLED_HEIGHT 64

Adafruit_SSD1306 oled(OLED_WIDTH, OLED_HEIGHT);

void setup() {
  if (!oled.begin(SSD1306_SWITCHCAPVCC, OLED_I2C_ADDRESS)) {
    while (true);
  }
  oled.clearDisplay();
  oled.display();
  randomSeed(analogRead(2));
}

void loop() {
  showRect();
  showRoundRect();
  showCircle();
  showLine();
  showTriangle();
  showPixel();
}

void showHeader(String text, bool filled) {
  oled.clearDisplay();
  oled.setTextSize(1);
  if (filled) {
    oled.fillRect(0, 0, 128, 16, WHITE);
    oled.setTextColor(BLACK);    
  } else {
    oled.drawLine(0, 16, 128, 16, WHITE);
    oled.setTextColor(WHITE);
  }
  oled.setCursor(OLED_WIDTH / 2, 8);
  printCenteredText(text);
  oled.display();
  delay(400);
}

void showRect() {
  showHeader("Rettangoli", true);
  
  oled.drawRect(31, 22, 60, 30, WHITE);
  oled.display();
  delay(400);
  
  oled.fillRect(38, 29, 60, 30, WHITE);
  oled.display();
  delay(400);
}

void showRoundRect() {
  showHeader("Bordi arrotondati", false);
  
  for (int r = 0; r < 15; r++) {
    oled.fillRect(0, 17, 128, 48, BLACK);
    oled.drawRoundRect(31, 22, 60, 30, r, WHITE);
    oled.display();
  }
  
  for (int r = 0; r < 15; r++) {
    oled.fillRect(0, 17, 128, 48, BLACK);
    oled.drawRoundRect(31, 22, 60, 30, 15, WHITE);
    oled.fillRoundRect(38, 29, 60, 30, r, WHITE);
    oled.display();
  }
  
  delay(200);
}

void showCircle() {
  showHeader("Cerchi", true);
  
  oled.drawCircle(54, 39, 15, WHITE);
  oled.display();
  delay(400);
  
  oled.fillCircle(74, 45, 15, WHITE);
  oled.display();
  delay(400);
}

void showLine() {
  showHeader("Linee", false);
  
  oled.drawLine(20, 30, 35, 50, WHITE);
  oled.display();
  delay(250);

  oled.drawLine(40, 37, 105, 28, WHITE);
  oled.display();
  delay(250);

  oled.drawLine(80, 45, 110, 56, WHITE);
  oled.display();
  delay(250);
}

void showTriangle() {
  showHeader("Triangoli", true);

  oled.drawTriangle(25, 54, 55, 20, 85, 48, WHITE);
  oled.display();
  delay(400);
  
  oled.fillTriangle(45, 58, 75, 24, 105, 52, WHITE);
  oled.display();
  delay(400);
}

void showPixel() {
  showHeader("Pixel", false);

  for (int i = 0; i < 50; i++) {
    oled.drawPixel(random(0, 128), random(20, 64), WHITE);
    oled.display();
  }
  
  delay(200);
}

void printCenteredText(String text) {
  int16_t x = 0, y = 0;
  uint16_t w = 0, h = 0;
  int16_t cursorX = oled.getCursorX();
  int16_t cursorY = oled.getCursorY();
  oled.getTextBounds(text, 0, 0, &x, &y, &w, &h);
  oled.setCursor(cursorX - x - w / 2, cursorY - y - h / 2);
  oled.print(text);
}